import {ThreadChannel} from 'discord.js';
import {addThread, addThreadMessage} from './modules/thread';
import {BotContext} from './common';
import {setMetadata} from './modules/channel';
import {isThreadInSelectedForumChannel} from './modules/context';

export const addMessageEventsListener = (context: BotContext) => {
	context.onThreadCreate(async (thread) => {
		if (typeof context.channel?.id === 'undefined') {
			return;
		}

		const isDescendant = await isThreadInSelectedForumChannel(context, thread);
		if (!isDescendant) {
			return;
		}

		await addThread(context.channel, thread);
		await setMetadata(context.channel);
	});

	context.onMessageCreate(async (message) => {
		if (typeof context.channel?.id === 'undefined') {
			return;
		}

		const thread = await context.fetchChannel(message.channelId);
		if (!thread) {
			return;
		}

		if (!(thread instanceof ThreadChannel)) {
			return;
		}

		const isDescendant = await isThreadInSelectedForumChannel(context, thread)
		if (!isDescendant) {
			return;
		}

		await addThreadMessage(context.channel, thread, message);
		await setMetadata(context.channel);
	});
};
