import {createClient} from './client';
import * as config from './config';
import {addMessageEventsListener} from './events';
import {ForumChannel} from 'discord.js';
import {setupThreads} from './modules/thread';
import {BotContext} from './common';

const main = async () => {
	const client = createClient({
		intents: config.meta.intentsInteger,
	});

	const context: BotContext = {
		channel: undefined,
		onMessageCreate: (fn) => {
			client.on('messageCreate', fn);
		},
		onThreadCreate: (fn) => {
			client.on('threadCreate', fn);
		},
		fetchChannel: (channelId: string) => client.channels.fetch(channelId),
	};

	client.on('ready', async () => {
		if (!client.user) {
			console.error('Not logged in!');
			return;
		}
		console.log(`Logged in as ${client.user.tag}`);

		const channel = await client.channels.fetch(config.meta.channelId);
		if (!(channel instanceof ForumChannel)) {
			console.error(`Channel ${config.meta.channelId} is not a forum!`);
			return;
		}

		context.channel = channel;
		await setupThreads(context);
	});

	addMessageEventsListener(context);
	await client.login(config.meta.token);
}

// TODO load channelId from config
// TODO add discord wizard for chatting with bot for setup (e.g. selecting forum channel ID)
void main();
