import {ForumChannel} from 'discord.js';
import {mkdir, readFile, rm, stat, writeFile} from 'fs/promises';

export const getChannelIdPath = (channel: ForumChannel) => `.db/${channel.id}`;

const getMetadataPath = (channel: ForumChannel) => `${getChannelIdPath(channel)}/.metadata.json`;

export const ensureChannelContent = async (channel: ForumChannel) => {
	try {
		await mkdir(getChannelIdPath(channel), {recursive: true});
	} catch {
		// noop
	}
};

export const getMetadata = async (channel: ForumChannel) => {
	const filePath = getMetadataPath(channel);
	let metadata;
	try {
		const s = await stat(filePath);
		if (s.isFile()) {
			const metadataJson = await readFile(filePath, 'utf-8');
			metadata = JSON.parse(metadataJson);

			// todo use valibot?
			if ('lastSyncedAt' in metadata) {
				return metadata;
			}
		}
	} catch {
		// noop
	}

	metadata = {
		lastSyncedAt: undefined,
	};

	await ensureChannelContent(channel);
	await rm(filePath, { recursive: true, force: true });
	await writeFile(filePath, JSON.stringify(metadata));
	return metadata;
};

export const setMetadata = async (channel: ForumChannel) => {
	const filePath = getMetadataPath(channel);
	await ensureChannelContent(channel);
	await writeFile(filePath, JSON.stringify({
		lastSyncedAt: Date.now(),
	}));
};
